-- Scaletron.applescript
-- Scaletron

--  Created by RJay Hansen on Sat Aug 09 2003.
-- Modified by RJay Hansen on Friday Oct. 27, 2006.
--  Copyright (c) 2006 __RJay Hansen__. All rights reserved.

property whichWindow : "version2" --property to hold our window preference
property keepPosition : 0 --remember window position?
property rezWindowPosition : missing value --these two get the window(s) bounds
property norezWindowPosition : missing value
property theWindow : missing value

property debugMode : false --set to false or comment out for no debugging
--property theDate : current date
--property expirationDate : date "Thursday, November 30, 2006 12:00:00 AM"
--The two previous properties were to create an expiration date for the beta
(*initialization stuff. set up default preferences, read preferences, open the chosen window (rez window is default) and get window's saved position and open there if chosen in preferences*)
on will finish launching theObject
	
	try
		setDefaultPrefs()
	on error
		display dialog "can't set prefs"
	end try
	getRunStatus("setting defaults")
	
	--set contents of default entry "defaultInterface" of user defaults to "version2"
	try
		readPrefs()
	on error
		display dialog "can't read prefs"
	end try
	getRunStatus("reading prefs")
	
	try
		positionWindow()
	on error
		display dialog "can't position window"
	end try
	
	try
		getWindow()
	on error
		display dialog "can't get window"
	end try
	
end will finish launching


--debugging handler; logs events
on getRunStatus(theMessage)
	if debugMode = true then
		log theMessage
	end if
end getRunStatus

--handler to set default preferences
on setDefaultPrefs()
	make new default entry at end of default entries of user defaults with properties {name:"defaultInterface", contents:"version2"}
	make new default entry at end of default entries of user defaults with properties {name:"rememberPos", contents:"0"}
	make new default entry at end of default entries of user defaults with properties {name:"rezWindPos", contents:{}}
	make new default entry at end of default entries of user defaults with properties {name:"norezWindPos", contents:{}}
	
	call method "synchronize" of object user defaults
end setDefaultPrefs

--handler to read in the preferences
on readPrefs()
	
	set whichWindow to contents of default entry "defaultInterface" of user defaults
	set keepPosition to contents of default entry "rememberPos" of user defaults
	set rezWindowPosition to contents of default entry "rezWindPos" of user defaults
	set norezWindowPosition to contents of default entry "norezWindPos" of user defaults
	
end readPrefs

--handler to save preferences set in pref window
on savePrefs()
	tell window "prefwindow"
		(*set userPrefs to {}
		set userPrefs to ((userPrefs & state of button "userez") & state of button "savebounds")*)
		set theInterface to state of button "userez"
		set keepPosition to state of button "savebounds" --as boolean
		
	end tell
	
	if theInterface = 1 then
		set whichWindow to "version2"
	else
		set whichWindow to "version1"
	end if
	
	tell user defaults
		set contents of default entry "defaultInterface" to whichWindow
		set contents of default entry "rememberPos" to keepPosition
	end tell
	
	call method "synchronize" of object user defaults
	
end savePrefs

--handler to get and position window
on getWindow()
	
	try
		
		--expiration for beta
		--if theDate  expirationDate then
		--show window "expirewindow"
		
		if whichWindow = "version2" then
			show window "rezwindow"
			hide window "norezwindow"
		else if whichWindow = "version1" then
			hide window "rezwindow"
			show window "norezwindow"
		end if
	on error
		display dialog "can't get default window."
	end try
	
	getRunStatus("getting window")
end getWindow

on positionWindow()
	try
		if keepPosition = 1 then
			set position of window "rezwindow" to rezWindowPosition
			set position of window "norezwindow" to norezWindowPosition
			
			my getRunStatus("setting window  position")
		end if
	on error
		display dialog "can't set window position"
	end try
end positionWindow

on saveWindowPoz()
	--get and save our window position. do required offsetting to keep windows aligned.
	if theWindow = missing value then
		set theWindow to main window
	end if
	--set theWindow to name of main window
	display dialog "the main window is " & theWindow
end saveWindowPoz

--a rounding handler (courtesy Bill Briggs from MacScpt mail list)
--first number is the number to be rounded, second number is the number of decimal places you want
on myRound(x, n)
	return (round x * (10 ^ n) rounding down) / (10 ^ n)
	--set roundedNum to result
end myRound

--here's our percentage handlers
--the first one calculates the percentage
on doPercent(num1, num2)
	num2 / num1 * 100
	return result
	--set thePercent to result
end doPercent

--This one calculates the new size of an item
on doResize(thePercent, theNum)
	thePercent / 100 * theNum
	return result
end doResize

--and the resolution handler
on doRez(theRez, thePercent)
	theRez div (thePercent / 100)
	return result
end doRez

on doCalculation()
	--if theWindow = missing value then
	set theWindow to main window
	--end if
	tell theWindow
		set oWidth to contents of text field "owidth" of theWindow
		set nWidth to contents of text field "nwidth" of theWindow
		set oHeight to contents of text field "oheight" of theWindow
		set nHeight to contents of text field "nheight" of theWindow
		set thePercent to contents of text field "percent" of theWindow
		if exists text field "oppi" of theWindow then
			set oPpi to contents of text field "oppi" of theWindow
		else
			set oPpi to ""
		end if
		if exists text field "nppi" of theWindow then
			set nPpi to contents of text field "nppi" of theWindow
		else
			set nPpi to ""
		end if
	end tell
	
	--check for width fields only and run percentage if true
	if oWidth  "" and nWidth  "" and oHeight = "" and oPpi = "" then
		--get percentage of original for new width
		doPercent(oWidth, nWidth)
		set thePercent to result
		myRound(thePercent, 4)
		set thePercent to result as string
		
		--update window
		--each window update has to check for the new rez field in case we're adressing the norez window
		tell theWindow
			set contents of text field "percent" to thePercent
			set contents of text field "nheight" to ""
			if exists text field "nppi" then
				set contents of text field "nppi" to ""
			end if
		end tell
		
		--check for original width and height and new width
	else if oWidth  "" and nWidth  "" and oHeight  "" and oPpi = "" then
		
		--first get the new width percentage
		doPercent(oWidth, nWidth)
		set thePercent to result
		myRound(thePercent, 4)
		set thePercent to result as string
		
		--now calculate the new height at that percentage
		doResize(thePercent, oHeight)
		set newSize to result
		myRound(newSize, 4)
		set newHeightSize to result as string
		
		--update window
		tell theWindow
			set contents of text field "percent" to thePercent
			set contents of text field "nheight" to newHeightSize
			if exists text field "nppi" then
				set contents of text field "nppi" to ""
			end if
		end tell
		
		--check for original and new height fields only	
	else if oWidth = "" and oHeight  "" and nHeight  "" and oPpi = "" then
		
		--get the new height percentage
		doPercent(oHeight, nHeight)
		set thePercent to result
		myRound(thePercent, 4)
		set thePercent to result as string
		
		--update window
		tell theWindow
			set contents of text field "percent" to thePercent
			set contents of text field "nwidth" to ""
			if exists text field "nppi" then
				set contents of text field "nppi" to ""
			end if
		end tell
		
		--check for original width, original height and new height	
	else if oWidth  "" and nWidth = "" and oHeight  "" and nHeight  "" and oPpi = "" then
		
		--get new height percentage
		doPercent(oHeight, nHeight)
		set thePercent to result
		myRound(thePercent, 4)
		set thePercent to result as string
		
		--and calculate the new width at that percentage
		doResize(thePercent, oWidth)
		set newSize to result
		myRound(newSize, 4)
		set newWidthSize to result as string
		
		--update window
		tell theWindow
			set contents of text field "percent" to thePercent
			set contents of text field "nwidth" to newWidthSize
			if exists text field "nppi" then
				set contents of text field "nppi" to ""
			end if
		end tell
		
		--check for original and new width and for original resolution
	else if oWidth  "" and nWidth  "" and oHeight = "" and oPpi  "" then
		
		--get new width percentage
		doPercent(oWidth, nWidth)
		set thePercent to result
		myRound(thePercent, 4)
		set thePercent to result as string
		
		--calculate new resolution
		tell theWindow
			if exists text field "oppi" then
				my doRez(oPpi, thePercent)
				set newRez to result as string
			end if
			
			--update window
			set contents of text field "percent" to thePercent
			if exists text field "nppi" then
				set contents of text field "nppi" to newRez
			end if
			set contents of text field "nheight" to ""
		end tell
		
		--check for original width and height and resolution and new width
	else if oWidth  "" and nWidth  "" and oHeight  "" and oPpi  "" then
		--first get the new width percentage
		doPercent(oWidth, nWidth)
		set thePercent to result
		myRound(thePercent, 4)
		set thePercent to result as string
		
		--now calculate the new height at that percentage
		doResize(thePercent, oHeight)
		set newSize to result
		myRound(newSize, 4)
		set newHeightSize to result as string
		
		--and calculate the new resolution
		tell theWindow
			if exists text field "oppi" then
				my doRez(oPpi, thePercent)
				set newRez to result as string
			end if
			
			--update window
			set contents of text field "percent" to thePercent
			set contents of text field "nheight" to newHeightSize
			if exists text field "nppi" then
				set contents of text field "nppi" to newRez
			end if
		end tell
		
		--check for original width and height and resolution and a new height
	else if oWidth  "" and nWidth = "" and oHeight  "" and nHeight  "" and oPpi  "" then
		--get new height percentage
		doPercent(oHeight, nHeight)
		set thePercent to result
		myRound(thePercent, 4)
		set thePercent to result as string
		
		--and calculate the new width at that percentage
		doResize(thePercent, oWidth)
		set newSize to result
		myRound(newSize, 4)
		set newWidthSize to result as string
		
		--and get the new resolution
		tell theWindow
			if exists text field "nppi" then
				my doRez(oPpi, thePercent)
				set newRez to result as string
			end if
			
			--update window
			set contents of text field "percent" to thePercent
			set contents of text field "nwidth" to newWidthSize
			if exists text field "nppi" then
				set contents of text field "nppi" to newRez
			end if
		end tell
		
		--original height and resolution and new height
	else if oWidth = "" and oHeight  "" and nHeight  "" and oPpi  "" then
		--get the new height percentage
		doPercent(oHeight, nHeight)
		set thePercent to result
		myRound(thePercent, 4)
		set thePercent to result as string
		
		--calculate new resolution
		tell theWindow
			if exists text field "oppi" then
				my doRez(oPpi, thePercent)
				set newRez to result as string
			end if
			
			--update window
			set contents of text field "percent" to thePercent
			set contents of text field "nwidth" to ""
			if exists text field "nppi" then
				set contents of text field "nppi" to newRez
			end if
		end tell
		
		--original height and resolution and new height
	else if oWidth = "" and oHeight  "" and nHeight  "" and oPpi  "" then
		--get the new height percentage
		doPercent(oHeight, nHeight)
		set thePercent to result
		myRound(thePercent, 4)
		set thePercent to result as string
		
		--calculate new resolution
		tell theWindow
			if exists text field "oppi" then
				my doRez(item 6 of theValues, thePercent)
				set newRez to result as string
			end if
			
			--update window
			set contents of text field "percent" to thePercent
			set contents of text field "nwidth" to ""
			if exists text field "nppi" then
				set contents of text field "nppi" to newRez
			end if
		end tell
		
		--check for original and new resolution only
	else if oWidth = "" and oHeight = "" and oPpi  "" and nPpi  "" then
		--get the new resolution percentage
		doPercent(oPpi, nPpi)
		set thePercent to result
		myRound(thePercent, 4)
		set thePercent to result as string
		
		--update window
		tell theWindow
			set contents of text field "percent" to thePercent
			set contents of text field "nwidth" to ""
			set contents of text field "nheight" to ""
		end tell
		
		--now we calculate new sizes for a given percentage
		--first check for original width and height and a percentage
	else if oWidth  "" and nWidth = "" and oHeight  "" and nHeight = "" and thePercent  "" and oPpi = "" then
		--calculate new size for width and heighth
		doResize(thePercent, oWidth)
		set newWidth to result
		myRound(newWidth, 4)
		set newWidth to result as string
		doResize(thePercent, oHeight)
		set newHeight to result
		myRound(newHeight, 4)
		set newHeight to result as string
		
		--update window
		tell theWindow
			set contents of text field "nwidth" to newWidth
			set contents of text field "nheight" to newHeight
			if exists text field "nppi" then
				set contents of text field "nppi" to ""
			end if
		end tell
		
		--check for an original width only and a percentage
	else if oWidth  "" and nWidth = "" and oHeight = "" and nHeight = "" and thePercent  "" and oPpi = "" then
		--Calculate the new width at given percentage
		doResize(thePercent, oWidth)
		set newWidth to result
		myRound(newWidth, 4)
		set newWidth to result as string
		
		--update window
		tell theWindow
			set contents of text field "nwidth" to newWidth
			set contents of text field "nheight" to ""
			if exists text field "nppi" then
				set contents of text field "nppi" to ""
			end if
		end tell
		
		--check for original height only and a percentage
	else if oWidth = "" and oHeight  "" and nHeight = "" and thePercent  "" and oPpi = "" then
		--calculate new height
		doResize(thePercent, oHeight)
		set newHeight to result
		myRound(newHeight, 4)
		set newHeight to result as string
		
		--update window
		tell theWindow
			set contents of text field "nheight" to newHeight
			set contents of text field "nwidth" to ""
			if exists text field "nppi" then
				set contents of text field "nppi" to ""
			end if
		end tell
		
		--check for original width and original resolution and percentage
	else if oWidth  "" and nWidth = "" and oHeight = "" and thePercent  "" and oPpi  "" and nPpi = "" then
		--calculate new width
		doResize(thePercent, oWidth)
		set newWidth to result
		myRound(newWidth, 4)
		set newWidth to result as string
		
		--calculate new resolution
		tell theWindow
			if exists text field "oppi" then
				my doRez(oPpi, thePercent)
				set newRez to result as string
			end if
			
			--update window
			set contents of text field "nwidth" to newWidth
			if exists text field "nppi" then
				set contents of text field "nppi" to newRez
			end if
			set contents of text field "nheight" to ""
		end tell
		
		--check for original height and original resolution and percentage
	else if oWidth = "" and oHeight  "" and nHeight = "" and thePercent  "" and oPpi  "" and nPpi = "" then
		--calculate new height
		doResize(thePercent, oHeight)
		set newHeight to result
		myRound(newHeight, 4)
		set newHeight to result as string
		
		--calculate new resolution
		tell theWindow
			if exists text field "oppi" then
				my doRez(oPpi, thePercent)
				set newRez to result as string
			end if
			
			--update window
			set contents of text field "nheight" to newHeight
			if exists text field "nppi" then
				set contents of text field "nppi" to newRez
			end if
			set contents of text field "nwidth" to ""
		end tell
		
		--original width and original height and original resolution and percentage
	else if oWidth  "" and nWidth = "" and oHeight  "" and nHeight = "" and thePercent  "" and oPpi  "" and nPpi = "" then
		--calculate new width and height
		doResize(thePercent, oWidth)
		set newWidth to result
		myRound(newWidth, 4)
		set newWidth to result as string
		doResize(thePercent, oHeight)
		set newHeight to result
		myRound(newHeight, 4)
		set newHeight to result as string
		
		--and the new resolution
		tell theWindow
			if exists text field "oppi" then
				my doRez(oPpi, thePercent)
				set newRez to result as string
			end if
			
			--update window
			set contents of text field "nwidth" to newWidth
			set contents of text field "nHeight" to newHeight
			if exists text field "nppi" then
				set contents of text field "nppi" to newRez
			end if
		end tell
		
		--and check for resolution only
	else if oWidth = "" and oHeight = "" and thePercent  "" and oPpi  "" and nPpi = "" then
		--calculate new resolution
		tell theWindow
			if exists text field "oppi" then
				my doRez(oPpi, thePercent)
				set newRez to result as string
				
				--update window
				set contents of text field "nppi" to newRez
				set contents of text field "nheight" to ""
				set contents of text field "nwidth" to ""
			end if
		end tell
		
	end if
end doCalculation --Whew!!

--reset button handler
on doReset()
	if theWindow = missing value then
		set theWindow to main window
	end if
	tell theWindow
		set contents of text field "owidth" to ""
		set contents of text field "oheight" to ""
		set contents of text field "nwidth" to ""
		set contents of text field "nheight" to ""
		if exists text field "oppi" then
			set contents of text field "oppi" to ""
		end if
		if exists text field "nppi" then
			set contents of text field "nppi" to ""
		end if
		set contents of text field "percent" to ""
		--move user focus back to the first field
		set first responder to text field "owidth"
	end tell
end doReset

on clicked theObject
	
	--setting a variable for the main window lets us handle either window's buttons w/the same handler
	set theWindow to main window
	set objectName to name of theObject --debug
	getRunStatus("clicked " & objectName)
	
	--handle calculate button
	if name of theObject is "calculate" then
		tell theWindow
			--my doRezCalculation()
			my doCalculation()
		end tell
		
		--handle reset button--sets all fields to blank	
	else if name of theObject is "reset" then
		tell theWindow
			my doReset()
		end tell
		
		--save preferences if pref window is okayed
	else if name of theObject is "saveprefs" then
		--tell window "prefwindow"
		my savePrefs()
		my readPrefs()
		
		hide window "prefwindow"
		
		set theWindow to main window
		if name of theWindow = "rezwindow" then
			set thePosition to position of window "rezwindow"
			set position of window "norezwindow" to {} & item 1 of thePosition & ((item 2 of thePosition) + 40)
		else if name of theWindow = "norezwindow" then
			set thePosition to position of window "norezwindow"
			set position of window "rezwindow" to {} & item 1 of thePosition & ((item 2 of thePosition) - 40)
		end if
		
		my getWindow()
		--end tell
		
		--handle prefs cancel button
	else if name of theObject is "cancelprefs" then
		hide window "prefwindow"
		return
		
		--takes user to website--used in beta. may put in "about" window eventually.	
	else if name of theObject is "gotowebsite" then
		open location "http://www.rocketjam.com/scaletron.html"
		--quit is for the beta expiration.
		quit
		
	end if
end clicked

--checking for invalid input
on should end editing theObject
	try
		set fieldContents to content of theObject as real
		return true
	on error
		display alert "Numbers only please!" attached to window "rezwindow"
		return false
	end try
end should end editing

--now handle the control menu
on choose menu item theObject
	--run the calculate handler for calculate menu item
	if name of theObject is "calculatemnu" then
		
		doCalculation()
		--and do a reset for reset!
	else if name of theObject is "resetmnu" then
		
		doReset()
	else if name of theObject is "prefmnu" then
		show window "prefwindow"
		
	end if
end choose menu item

on will close theObject
	--get current window's position and save to preferences
	set currentWindowPosition to position of theObject
	if name of theObject = "rezwindow" then
		set norezWindowPosition to {} & item 1 of currentWindowPosition & ((item 2 of currentWindowPosition) + 40)
		
		tell user defaults
			set contents of default entry "rezWindPos" to currentWindowPosition
			set contents of default entry "norezWindPos" to norezWindowPosition
		end tell
		
	else if name of theObject = "norezwindow" then
		set rezWindowPosition to {} & item 1 of currentWindowPosition & ((item 2 of currentWindowPosition) - 40)
		
		tell user defaults
			set contents of default entry "norezWindPos" to currentWindowPosition
			set contents of default entry "rezWindPos" to rezWindowPosition
		end tell
	end if
	
	getRunStatus("Saving window position.")
	
end will close

--quit if close button is pushed
on should quit after last window closed theObject
	return true
end should quit after last window closed

--set state of pref window's checkboxes to state they were in when last used
on will open theObject
	if theObject = window "prefwindow" then
		if whichWindow = "version2" then
			tell window "prefwindow"
				set state of button "userez" to 1
			end tell
		end if
		if keepPosition = 1 then
			tell window "prefwindow"
				set state of button "savebounds" to 1
			end tell
		end if
	end if
end will open

